package RoutingMatrix;
import java.util.*;
import com.cycling74.max.*;
public class ConnectionLibrary {

	
	Connection[][] myLib;
	DSPInstruction[] dspInstructions;
	
	float[] inMin,inMax,outMin,outMax;
	DSPInstruction[] defaultValues;
	
	
	
	public ConnectionLibrary(int in,int out)
	{
		myLib= new Connection[in][out];
		
		inMin  = new float[in];
		inMax  = new float[in];
		outMin = new float[out];
		outMax = new float[out];
		defaultValues = new DSPInstruction[out];
		
		Arrays.fill(inMin, 0.f);
		Arrays.fill(inMax, 1.f);
		Arrays.fill(outMin, 0.f);
		Arrays.fill(outMax, 1.f);
		
		Arrays.fill(defaultValues,new Sig(0.f));
		
		for(int i = 0 ; i < myLib.length;i++)
		{
			for(int j=0;j<myLib[0].length;j++)
			{
			
					myLib[i][j] = new Connection(i,j,inMin[i],inMax[i],outMin[j],outMax[j],false);
				
			}
		}
		
	}
	
	
	
	
	public void setDefault(int outNumber,float f)
	{
		defaultValues[outNumber] = new Sig(f);
	}
	
	
	public DSPInstruction[] clearConnections()
	{
		
		int ins = myLib.length;
		int outs = myLib[0].length;
		
		for(int i = 0 ; i < myLib.length;i++)
		{
			for(int j=0;j<myLib[0].length;j++)
			{
				if(myLib[i][j]!=null)
				{
					addConnection(i,j);
					myLib[i][j].setOn(false);
				}
			
			}
		}
		updateLinearList();
		 
		return dspInstructions;
	}
	
	
	public void post()
	{
		post("");
	}
	public void post(String s)
	{
		MaxObject.post("CONNECTION_LIBRARY---    in/out"+myLib.length+"/"+myLib[0].length);
		for(int i = 0 ; i < myLib.length;i++)
		{
			for(int j=0;j<myLib[0].length;j++)
			{
				if(myLib[i][j].isOn())
				{
					myLib[i][j].post("i/j"+i+"/"+j);
				}
			
			}
		}
		
		
		postArray("inMin",inMin);
		postArray("inMax",inMax);
		postArray("outMin",outMin);
		postArray("outMax",outMax);
		
		
		
		
		
	}
	
	
	public DSPInstruction[] setInRange(int id, float min,float max)
	{
		inMin[id] = min;
		inMax[id] = max;
		
		
		for(int i = 0 ; i < myLib[0].length;i++)
		{
			if(myLib[id][i]!=null)
			{
				myLib[id][i].setInMin(min);
				myLib[id][i].setInMax(max);
			}
		}
		updateLinearList();
		 
		return dspInstructions;
	}
	
	
	public DSPInstruction[] setOutRange(int id, float min,float max)
	{
		outMin[id] = min;
		outMax[id] = max;
		
		
		for(int i = 0 ; i < myLib.length;i++)
		{
			if(myLib[i][id]!=null)
			{
				myLib[i][id].setOutMin(min);
				myLib[i][id].setOutMax(max);
			}
		}
		updateLinearList();
		
		return dspInstructions;
	}
	
	
	 
	public DSPInstruction[] addConnection(int in,int out)
	{
	//	MaxObject.post("ConLib addConnect in out"+in+"/"+out);
	//	myLib[in][out] = new Connection(in,out,inMin[in],inMax[in],outMin[out],outMax[out]);
		myLib[in][out].setOn(true);
	//	post();
		updateLinearList();
	//	post();
		return dspInstructions;
	}
	
	
	public DSPInstruction[] getDspInstructions()
	{
        updateLinearList();
		
		return dspInstructions;
	}
	
	public void updateLinearList()
	{
		
		int[] outQuantities = new int[myLib[0].length];
		Arrays.fill(outQuantities, 0);
		for(int i = 0 ; i < myLib.length;i++)
		{
			for(int j=0;j<myLib[0].length;j++)
			{
				if(myLib[i][j].isOn())
				{
					outQuantities[j]++;
					
				}
			}
		}
		
		int numOutsAssigned = 0;
		for(int i = 0 ; i < myLib[0].length;i++ )
		{
			if(outQuantities[i]>0)
			{
				numOutsAssigned ++;
			}
		}
		
		dspInstructions = defaultValues.clone();
	
		for (int j = 0; j < myLib[0].length; j++) {

			Connection[] thisOutsConnections = new Connection[outQuantities[j]];
			int runningNumThisOutConnections = 0;
			for (int i = 0; i < myLib.length; i++) {

				
			//	MaxObject.post(i+"/"+j+" is "+myLib[i][j].isOn()	);
				if (myLib[i][j].isOn()) {
					thisOutsConnections[runningNumThisOutConnections] = myLib[i][j];
					runningNumThisOutConnections++;
				}
			}
			
		//	MaxObject.post("j before setting instructions"+dspInstructions[j].getType());
			if (runningNumThisOutConnections != 0) {
				dspInstructions[j] = new OutConnectionLibrary(j, thisOutsConnections);	
				
			}
			
		}

		
	}

	
	public DSPInstruction[] setMode(int in,int out,int mode)
	{
		myLib[in][out].setMode(mode);
		updateLinearList();
		 
		return dspInstructions;
		
	}
	
	public DSPInstruction[] setAmount(int in,int out,float amt)
	{
		myLib[in][out].setAmount(amt);
		updateLinearList();
		 
		return dspInstructions;
		
	}
	
	public DSPInstruction[]  removeConnection(int in,int out)
	{
		myLib[in][out].setOn(false);
		post();
		updateLinearList();
		 
		return dspInstructions;
	}
	
	
	public void postArray(String start ,float[] array)
	{
		String temp = start+": ";
		for(int i = 0 ; i <  array.length;i++)
		{
			temp = temp+array[i]+" ";
		}
		
		MaxObject.post(temp);
	}
}
