package RoutingMatrix;

import com.cycling74.max.*;
import com.cycling74.msp.*;
import java.util.*;

public class RoutingMatrixMobj extends MSPPerformer {

	private static final String[] INLET_ASSIST = new String[] { "input (sig)" };
	private static final String[] OUTLET_ASSIST = new String[] { "output (sig)" };

	ConnectionLibrary myLib;
	DSPInstruction[] dspInstructions = new OutConnectionLibrary[0];

	int numIns = 0;
	int numOuts = 0;
	float defaultValues[];
	
	
	public RoutingMatrixMobj(int in, int out) {

		int[] inId = new int[in];
		Arrays.fill(inId, SIGNAL);
		int[] outId = new int[out];
		Arrays.fill(outId, SIGNAL);
		this.numIns = in;
		this.numOuts = out;

		declareInlets(inId);
		declareOutlets(outId);
		myLib = new ConnectionLibrary(in, out);
		dspInstructions = myLib.getDspInstructions();
		defaultValues = new float[out];
		Arrays.fill(defaultValues,0f);
		
	}

	public void inlet(float f) {

	}

	public void post() {
		MaxObject.post("MYLIB:");
		myLib.post();
		MaxObject.post("");
		MaxObject.post("my dsp instructions      length="+dspInstructions.length);
		for (int i = 0; i < dspInstructions.length; i++) {
			MaxObject.post("i"+i);
			switch (dspInstructions[i].getType()) {
			case DSPInstruction.CONNECTIONS:
				((OutConnectionLibrary)dspInstructions[i]).post("instruction["+i+"]");
				break;
			case DSPInstruction.SIG:
				((Sig)dspInstructions[i]).post("instruction["+i+"]");
				break;
			default:
				MaxObject.post("default?");
			break;
			}
		}

	}

	public void addConnection(int in, int out) {
		dspInstructions = myLib.addConnection(in, out);
	}
	
	public void removeConnection(int in,int out)
	{
		dspInstructions = myLib.removeConnection(in, out);
	}

	public void setInRange(int id, float min, float max) {
		dspInstructions = myLib.setInRange(id, min, max);
	}

	public void setOutRange(int id, float min, float max) {
		dspInstructions = myLib.setOutRange(id, min, max);
	}

	public void dspsetup(MSPSignal[] ins, MSPSignal[] outs) {
	
		
	}

	public void setDefault(int id, float value)
	{
		defaultValues[id]=value;	
	}
	
	public void setMode(int in, int out, int mode)
	{
		dspInstructions= myLib.setMode(in, out, mode);
	}
	
	public void setAmount(int in, int out, float amt)
	{
		dspInstructions= myLib.setAmount(in, out, amt);
	}
	
	
	
	public void perform(MSPSignal[] ins, MSPSignal[] outSigs) {

		// MaxObject.post(".");

		for (int currOut = 0; currOut < numOuts; currOut++) {
			switch (dspInstructions[currOut].getType()) {
			case DSPInstruction.CONNECTIONS:
				
				OutConnectionLibrary currOutsLib = (OutConnectionLibrary)dspInstructions[currOut];
				
				Arrays.fill(outSigs[currOut].vec, defaultValues[currOut]);

				for (int currInToThisOutCounter = 0; currInToThisOutCounter < currOutsLib.getConnections().length; currInToThisOutCounter++) {
					int inToThisOut = currOutsLib.getConnections()[currInToThisOutCounter].in;
					float mult = currOutsLib.getConnections()[currInToThisOutCounter].getMult();
					float add = currOutsLib.getConnections()[currInToThisOutCounter].getAdd();

					// MaxObject.post("currOut"+currOut+"    inToThisOut"+inToThisOut);
					for (int i = 0; i < ins[0].n; i++) {
						outSigs[currOut].vec[i] = outSigs[currOut].vec[i] += (ins[inToThisOut].vec[i]
								* mult + add);
					}

				}
				break;
			case DSPInstruction.SIG:
			
				for (int i = 0; i < ins[0].n; i++) {
					outSigs[currOut].vec[i] = defaultValues[currOut];
				}

			break;
			}

		}

	}
}
